package main.java.config;

import javax.sql.DataSource;
import org.apache.commons.dbcp2.BasicDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.DefaultServletHandlerConfigurer;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;
import org.springframework.web.servlet.view.InternalResourceViewResolver;

import main.java.dao.AccountDao;
import main.java.dao.AccountDaoImpl;
import main.java.dao.BookDao;
import main.java.dao.BookDaoImpl;
import main.java.dao.CirculationDao;
import main.java.dao.CirculationDaoImpl;
import main.java.dao.ConnectionInfo;
import main.java.util.ProcessingTimeInterceptor;

@Configuration
@ComponentScan("main.java")
@EnableWebMvc
public class ApplicationContextConfig extends WebMvcConfigurerAdapter {

	@Autowired
	private ProcessingTimeInterceptor processingTimeInterceptor;

	@Bean(name = "viewResolver")
	public InternalResourceViewResolver getViewResolver() {
		InternalResourceViewResolver viewResolver = new InternalResourceViewResolver();
		viewResolver.setPrefix("/WEB-INF/views/");
		viewResolver.setSuffix(".jsp");
		return viewResolver;
	}

	@Override
	public void configureDefaultServletHandling(DefaultServletHandlerConfigurer configurer) {
		configurer.enable();
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(processingTimeInterceptor);
	}

	@Bean(name = "processingTimeInterceptor")
	public ProcessingTimeInterceptor getProcessingTimeInterceptor() {
		return new ProcessingTimeInterceptor();
	}

	@Bean(name = "dataSource")
	public DataSource getDataSource() {
		BasicDataSource dataSource = new BasicDataSource();
		dataSource.setDriverClassName("com.mysql.jdbc.Driver");
		dataSource.setUrl("jdbc:mysql://192.168.3.1:3306/SpringLibrary");
		dataSource.setUsername("SpringLibrary");
		dataSource.setPassword("xqawtNucWNYaLpQu");
		return dataSource;
	}

	@Bean(name = "connectionInfo")
	public ConnectionInfo getConnection() {
		return new ConnectionInfo();
	}

	@Autowired
	@Bean(name = "accountDao")
	public AccountDao getAccountDao(ConnectionInfo connectionInfo) {
		return new AccountDaoImpl(connectionInfo);
	}

	@Autowired
	@Bean(name = "bookDao")
	public BookDao getBookDao(ConnectionInfo connectionInfo) {
		return new BookDaoImpl(connectionInfo);
	}

	@Autowired
	@Bean(name = "circulationDao")
	public CirculationDao getcirculationDao(ConnectionInfo connectionInfo) {
		return new CirculationDaoImpl(connectionInfo);
	}
}