package main.java.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import com.mysql.jdbc.exceptions.jdbc4.MySQLIntegrityConstraintViolationException;

import main.java.dao.AccountDao;
import main.java.model.Account;

@Controller
@RequestMapping("/account")
public class AccountController {

	@Autowired
	private AccountDao accountDao;

	@RequestMapping(value = "/", method = RequestMethod.GET)
	public ModelAndView index() {
		List<Account> accountList = accountDao.listEager();
		ModelAndView model = new ModelAndView("accountList");
		model.addObject("accountList", accountList);
		return model;
	}

	@RequestMapping(value = "/add/", method = RequestMethod.GET)
	public ModelAndView addAccount() {
		ModelAndView model = new ModelAndView("formAccount");
		model.addObject("account", new Account());
		model.addObject("accountButtonText", "Benutzer hinzufügen");
		return model;
	}

	@RequestMapping(value = "/add/confirmed/", method = RequestMethod.POST)
	public ModelAndView saveAccount(@Valid Account account, BindingResult result) {
		if (result.hasErrors()) {
			ModelAndView model = new ModelAndView("formAccount");
			model.addObject("account", account);
			return model;
		}
		accountDao.save(account);
		return new ModelAndView("redirect:/account/");
	}

	@RequestMapping(value = "/edit/{id}/", method = RequestMethod.GET)
	public ModelAndView editAccount(@PathVariable("id") int accountID, HttpServletRequest request) {
		Account account = accountDao.get(accountID);
		ModelAndView model = new ModelAndView("formAccount");
		model.addObject("account", account);
		model.addObject("edit", true);
		return model;
	}

	@RequestMapping(value = "/edit/confirmed/", method = RequestMethod.POST)
	public ModelAndView editAccountConfirmed(@Valid Account account, BindingResult result) {
		if (result.hasErrors()) {
			ModelAndView model = new ModelAndView("formAccount");
			model.addObject("account", account);
			model.addObject("edit", true);
			return model;
		}
		accountDao.update(account);
		return new ModelAndView("redirect:/account/");
	}

	@RequestMapping(value = "/delete/{id}/", method = RequestMethod.GET)
	public ModelAndView deleteAccount(@PathVariable("id") int accountID, HttpServletRequest request) {
		Account account = accountDao.getEager(accountID);
		if (account.hasCirculations()) {
			ModelAndView model = new ModelAndView("exception");
			model.addObject("exception", "Der Benutzer kann nicht gelöscht werden, da er noch Bücher ausgeliehen hat.");
			model.addObject("redirect", "account/");
			return model;
		}
		ModelAndView model = new ModelAndView("accountDelete");
		model.addObject("account", account);
		return model;
	}

	@RequestMapping(value = "/delete/{id}/confirmed/", method = RequestMethod.GET)
	public ModelAndView deleteAccountConfirmed(@PathVariable("id") int accountID, HttpServletRequest request) {
		try {
			accountDao.delete(accountID);
		} catch (RuntimeException e) {
			if (e.getCause().getClass() == MySQLIntegrityConstraintViolationException.class) {
				ModelAndView model = new ModelAndView("exception");
				model.addObject("exception",
						"Der Benutzer kann nicht gelöscht werden, da er noch Bücher ausgeliehen hat.");
				model.addObject("redirect", "account/");
				return model;
			}
		}
		return new ModelAndView("redirect:/account/");
	}

}
