package main.java.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import com.mysql.jdbc.exceptions.jdbc4.MySQLIntegrityConstraintViolationException;

import main.java.dao.BookDao;
import main.java.model.Book;

@Controller
@RequestMapping("/book")
public class BookController {

	@Autowired
	private BookDao bookDao;

	@RequestMapping(value = "/", method = RequestMethod.GET)
	public ModelAndView index() {
		List<Book> bookList = bookDao.list();
		ModelAndView model = new ModelAndView("bookList");
		model.addObject("bookList", bookList);
		return model;
	}

	@RequestMapping(value = "/add/", method = RequestMethod.GET)
	public ModelAndView addBook() {
		ModelAndView model = new ModelAndView("formBook");
		model.addObject("book", new Book());
		return model;
	}

	@RequestMapping(value = "/add/confirmed/", method = RequestMethod.POST)
	public ModelAndView addBookConfirmed(@Valid Book book, BindingResult result) {
		try {
			if (result.hasErrors()) {
				ModelAndView model = new ModelAndView("formBook");
				model.addObject("book", book);
				return model;
			}
			bookDao.save(book);
		} catch (RuntimeException e) {
			if (e.getCause().getClass() == MySQLIntegrityConstraintViolationException.class) {
				ModelAndView model = new ModelAndView("formBook");
				model.addObject("book", book);
				result.addError(
						new FieldError(result.getObjectName(), "isbn", "ISBN bereits in der Datenbank vorhanden!"));
				return model;
			}
		}
		return new ModelAndView("redirect:/book/");
	}

	@RequestMapping(value = "/edit/{id}/", method = RequestMethod.GET)
	public ModelAndView editBook(@PathVariable("id") String isbn, HttpServletRequest request) {
		Book book = bookDao.get(isbn);
		ModelAndView model = new ModelAndView("formBook");
		model.addObject("book", book);
		model.addObject("edit", true);
		return model;
	}

	@RequestMapping(value = "/edit/confirmed/", method = RequestMethod.POST)
	public ModelAndView editBookConfirmed(@Valid Book book, BindingResult result) {
		if (result.hasErrors()) {
			ModelAndView model = new ModelAndView("formBook");
			model.addObject("book", book);
			model.addObject("edit", true);
			return model;
		}
		bookDao.update(book);
		return new ModelAndView("redirect:/book/");
	}

	@RequestMapping(value = "/delete/{id}/", method = RequestMethod.GET)
	public ModelAndView deleteBook(@PathVariable("id") String isbn) {
		Book book = bookDao.getEager(isbn);
		if (book.hasCirculations()) {
			ModelAndView model = new ModelAndView("exception");
			model.addObject("exception", "Das Buch kann nicht gelöscht werden, da es noch ausgeliehen ist.");
			model.addObject("redirect", "book/");
			return model;
		}
		ModelAndView model = new ModelAndView("bookDelete");
		model.addObject("book", book);
		return model;
	}

	@RequestMapping(value = "/delete/{id}/confirmed/", method = RequestMethod.GET)
	public ModelAndView deleteBookConfirmed(@PathVariable("id") String isbn) {
		try {
			bookDao.delete(isbn);
		} catch (RuntimeException e) {
			if (e.getCause().getClass() == MySQLIntegrityConstraintViolationException.class) {
				ModelAndView model = new ModelAndView("exception");
				model.addObject("exception", "Das Buch kann nicht gelöscht werden, da es noch ausgeliehen ist.");
				model.addObject("redirect", "book/");
				return model;
			}
		}
		return new ModelAndView("redirect:/book/");
	}
}
