package main.java.controller;

import java.util.Collections;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import main.java.dao.AccountDao;
import main.java.dao.BookDao;
import main.java.dao.CirculationDao;
import main.java.model.Account;
import main.java.model.Book;
import main.java.model.Circulation;

@Controller
@RequestMapping("/circulation")
public class CirculationController {

	@Autowired
	private AccountDao accountDao;
	@Autowired
	private BookDao bookDao;
	@Autowired
	private CirculationDao circulationDao;

	@RequestMapping(value = "/", method = RequestMethod.GET)
	public ModelAndView index() {
		List<Circulation> circulationList = circulationDao.list();
		ModelAndView model = new ModelAndView("circulationList");
		Collections.sort(circulationList, (x, y) -> x.getLendDate().compareTo(y.getLendDate()));
		model.addObject("circulationList", circulationList);
		return model;
	}

	@RequestMapping(value = "/add/{isbn}/", method = RequestMethod.GET)
	public ModelAndView add(@PathVariable("isbn") String bookID) {
		if (bookDao.get(bookID).getStock() == 0) {
			ModelAndView model = new ModelAndView("exception");
			model.addObject("exception", "Alle Exemplare dieses Buches sind momentan ausgeliehen!");
			model.addObject("redirect", "book/");
			return model;
		}
		List<Account> accountList = accountDao.listEagerNotLendBook(bookID);
		if (accountList.isEmpty()) {
			ModelAndView model = new ModelAndView("exception");
			model.addObject("exception", "Alle Benutzer haben dieses Buch bereits ausgeliehen.");
			model.addObject("redirect", "book/");
			return model;
		}
		ModelAndView model = new ModelAndView("selectAccount");
		model.addObject("accountList", accountList);
		model.addObject("bookID", bookID);
		return model;
	}

	@RequestMapping(value = "/lend/{isbn}/", method = RequestMethod.POST)
	public ModelAndView lend(@PathVariable("isbn") String bookID, @RequestParam(value = "accountID") int accountID) {
		circulationDao.save(bookID, accountID);
		return new ModelAndView("redirect:/account/");
	}

	@RequestMapping(value = "/return/{isbn}/{accountId}", method = RequestMethod.GET)
	public ModelAndView returnCirculation(@PathVariable("isbn") String isbn, @PathVariable("accountId") int accountId) {
		Book book = bookDao.get(isbn);
		ModelAndView model = new ModelAndView("returnCirculation");
		model.addObject("book", book);
		model.addObject("accoundId", accountId);
		return model;
	}

	@RequestMapping(value = "/return/{isbn}/{accountId}/confirmed/", method = RequestMethod.GET)
	public ModelAndView returnCirculationConfirmed(@PathVariable("isbn") String isbn,
			@PathVariable("accountId") int accountId) {
		circulationDao.update(isbn, accountId);
		return new ModelAndView("redirect:/account/");
	}
}
