package main.java.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import main.java.model.Account;
import main.java.model.Book;

public class AccountDaoImpl implements AccountDao {

	ConnectionInfo connectionInfo;
	@Autowired
	private BookDao bookDao;
	@Autowired
	private CirculationDao circulationDao;

	public AccountDaoImpl() {

	}

	public AccountDaoImpl(ConnectionInfo connectionInfo) {
		this.connectionInfo = connectionInfo;
	}

	@Override
	public List<Account> listEager() {
		List<Account> accounts = new ArrayList<Account>();
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement(
					"Select * from Account left join Circulation on Account.id = Circulation.accountId "
							+ "left join Book on Circulation.bookId = Book.isbn;");
			ResultSet rs = ps.executeQuery();
			accounts = fillListFromResultSet(rs);
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		return accounts;
	}

	@Override
	public List<Account> listEagerNotLendBook(String isbn) {
		List<Account> accounts = new ArrayList<Account>();
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Select * from Account "
					+ "left join Circulation on Account.id = Circulation.accountId "
					+ "left join Book on Circulation.bookId = Book.isbn where id not in (Select id from Account "
					+ "left join Circulation on Account.id = Circulation.accountId "
					+ "left join Book on Circulation.bookId = Book.isbn where (bookId Like ?));");
			ps.setString(1, isbn);
			ResultSet rs = ps.executeQuery();
			accounts = fillListFromResultSet(rs);
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		return accounts;
	}

	@Override
	public void save(Account account) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn
					.prepareStatement("INSERT INTO Account (id, email, password, username) Values (?,?,?,?);");
			ps.setInt(1, account.getId());
			ps.setString(2, account.getEmail());
			ps.setString(3, account.getPassword());
			ps.setString(4, account.getUsername());
			ps.executeUpdate();
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public void update(Account account) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn
					.prepareStatement("UPDATE Account SET email=?, password=?, username=?  Where id=?;");
			ps.setString(1, account.getEmail());
			ps.setString(2, account.getPassword());
			ps.setString(3, account.getUsername());
			ps.setInt(4, account.getId());
			ps.executeUpdate();
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public void delete(int id) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Delete from Account Where id=?;");
			ps.setInt(1, id);
			ps.executeUpdate();
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public Account get(int id) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Select * from Account Where id=?;");
			ps.setInt(1, id);
			ResultSet rs = ps.executeQuery();
			rs.next();
			Account account = fillFromResultSet(rs);
			conn.close();
			return account;
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public Account getEager(int id) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement(
					"Select * from Account left join Circulation on Account.id = Circulation.accountId left join Book on Circulation.bookId = Book.isbn where id=?;");
			ps.setInt(1, id);
			ResultSet rs = ps.executeQuery();
			List<Account> accounts = fillListFromResultSet(rs);
			if (accounts.isEmpty()) {
				throw new SQLException("Das Account wurde nicht gefunden!");
			}
			conn.close();
			return accounts.get(0);
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	public List<Account> fillListFromResultSet(ResultSet rs) throws SQLException {
		List<Account> accounts = new ArrayList<Account>();
		while (rs.next()) {
			rs.getInt("accountId");
			if (rs.wasNull()) {
				accounts.add(fillFromResultSet(rs));
			} else {
				Account accountFromDb = fillFromResultSet(rs);
				Book book = bookDao.fillFromResultSet(rs);
				if (accounts.contains(accountFromDb)) {
					Account account = accounts.get(accounts.indexOf(accountFromDb));
					account.addCirculation(circulationDao.fillFromResultSet(rs, book, account));
				} else {
					accountFromDb.addCirculation(circulationDao.fillFromResultSet(rs, book, accountFromDb));
					accounts.add(accountFromDb);
				}
			}
		}
		return accounts;
	}

	public Account fillFromResultSet(ResultSet rs) throws SQLException {
		Account account = new Account();
		account.setId(rs.getInt("id"));
		account.setEmail(rs.getString("email"));
		account.setPassword(rs.getString("password"));
		account.setUsername(rs.getString("username"));
		return account;
	}
}
