package main.java.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import main.java.model.Account;
import main.java.model.Book;

public class BookDaoImpl implements BookDao {

	ConnectionInfo connectionInfo;
	@Autowired
	private AccountDao accountDao;
	@Autowired
	private CirculationDao circulationDao;

	public BookDaoImpl(ConnectionInfo connectionInfo) {
		this.connectionInfo = connectionInfo;
	}

	@Override
	public List<Book> list() {
		List<Book> books = new ArrayList<Book>();
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Select * from Book;");
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				books.add(fillFromResultSet(rs));
			}
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		return books;
	}

	@Override
	public void save(Book book) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn
					.prepareStatement("INSERT INTO Book (isbn, description, stock, title) Values (?,?,?,?);");
			ps.setString(1, book.getIsbn());
			ps.setString(2, book.getDescription());
			ps.setInt(3, book.getStock());
			ps.setString(4, book.getTitle());
			ps.executeUpdate();
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public void update(Book book) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn
					.prepareStatement("UPDATE Book SET description=?, stock=?, title=?  Where isbn=?;");
			ps.setString(1, book.getDescription());
			ps.setInt(2, book.getStock());
			ps.setString(3, book.getTitle());
			ps.setString(4, book.getIsbn());
			ps.executeUpdate();
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public void delete(String isbn) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Delete from Book Where isbn=?;");
			ps.setString(1, isbn);
			ps.executeUpdate();
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public Book get(String isbn) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Select * from Book Where isbn=?;");
			ps.setString(1, isbn);
			ResultSet rs = ps.executeQuery();
			rs.next();
			Book book = fillFromResultSet(rs);
			conn.close();
			return book;
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	@Override
	public Book getEager(String isbn) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement(
					"Select * from Book " + "left join Circulation on Book.isbn = Circulation.bookId left "
							+ "join Account on Circulation.accountId = Account.id where isbn like ?;");
			ps.setString(1, isbn);
			ResultSet rs = ps.executeQuery();
			Book book = null;
			while (rs.next()) {
				if (book == null) {
					book = fillFromResultSet(rs);
				}
				rs.getString("bookId");
				if (!rs.wasNull()) {
					Account account = accountDao.fillFromResultSet(rs);
					book.addCirculation(circulationDao.fillFromResultSet(rs, book, account));
				}
			}
			if (book == null) {
				throw new SQLException("Das Buch wurde nicht gefunden!");
			}
			conn.close();
			return book;
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	public Book fillFromResultSet(ResultSet rs) throws SQLException {
		Book book = new Book();
		book.setIsbn(rs.getString("isbn"));
		book.setDescription(rs.getString("description"));
		book.setStock(rs.getInt("stock"));
		book.setTitle(rs.getString("title"));
		return book;
	}
}
