package main.java.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import main.java.model.Account;
import main.java.model.Book;
import main.java.model.Circulation;

public class CirculationDaoImpl implements CirculationDao {
	ConnectionInfo connectionInfo;

	@Autowired
	private BookDao bookDao;
	@Autowired
	private AccountDao accountDao;

	public CirculationDaoImpl(ConnectionInfo connectionInfo) {
		this.connectionInfo = connectionInfo;
	}

	@Override
	public List<Circulation> list() {
		List<Circulation> circulations = new ArrayList<Circulation>();
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement(
					"Select * From Circulation inner join Account on accountId=id inner join Book on bookId=isbn;");
			ResultSet rs = ps.executeQuery();
			while (rs.next()) {
				circulations
						.add(fillFromResultSet(rs, bookDao.fillFromResultSet(rs), accountDao.fillFromResultSet(rs)));
			}
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		return circulations;
	}

	@Override
	public void update(String isbn, int accountID) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement("Delete from Circulation where bookId=? and accountId=?;");
			ps.setString(1, isbn);
			ps.setInt(2, accountID);
			if (ps.executeUpdate() != 0) {
				PreparedStatement psNext = conn.prepareStatement("Update Book Set stock=stock + 1 where isbn like ?;");
				psNext.setString(1, isbn);
				psNext.executeUpdate();
			}
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage());
		}
	}

	@Override
	public void save(String isbn, int accountID) {
		try {
			Connection conn = connectionInfo.getConnection();
			PreparedStatement ps = conn.prepareStatement(
					"INSERT INTO Circulation (lendDate, returnDate, bookId, accountId) Values (?,?,?,?);");
			Instant now = Instant.now();
			ps.setTimestamp(1, new Timestamp(now.toEpochMilli()));
			ps.setTimestamp(2, new Timestamp(now.plus(14, ChronoUnit.DAYS).toEpochMilli()));
			ps.setString(3, isbn);
			ps.setInt(4, accountID);
			if (ps.executeUpdate() != 0) {
				PreparedStatement psNext = conn.prepareStatement("Update Book Set stock=stock - 1 where isbn like ?;");
				psNext.setString(1, isbn);
				psNext.executeUpdate();
			}
			conn.close();
		} catch (SQLException e) {
			throw new RuntimeException(e.getMessage());
		}
	}

	public Circulation fillFromResultSet(ResultSet rs, Book book, Account account) throws SQLException {
		Circulation circulation = new Circulation();
		circulation.setLendDate(rs.getTimestamp("lendDate").toInstant());
		circulation.setReturnDate(rs.getTimestamp("returnDate").toInstant());
		circulation.setAccount(account);
		circulation.setBook(book);
		return circulation;
	}
}
